@@ -1,3 +1,5 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
+
 /*
  *  (GLABELS) Label and Business Card Creation program for GNOME
  *
@@ -31,6 +33,7 @@
 #include "prefs.h"
 #include "mygal/widget-color-combo.h"
 #include "color.h"
+#include "util.h"
 
 #include "object-editor-private.h"
 
@@ -123,12 +126,7 @@ gl_object_editor_set_fill_color (glObjectEditor      *editor,
 
 	gl_debug (DEBUG_EDITOR, "START");
 
-	g_signal_handlers_block_by_func (G_OBJECT(editor->priv->fill_color_combo),
-					 G_CALLBACK (gl_object_editor_changed_cb),
-					 editor);
-	g_signal_handlers_block_by_func (G_OBJECT(editor->priv->fill_key_combo),
-					 G_CALLBACK (gl_object_editor_changed_cb),
-					 editor);
+        editor->priv->stop_signals = TRUE;
 
 	gtk_widget_set_sensitive (editor->priv->fill_key_radio, merge_flag);
 
@@ -161,12 +159,7 @@ gl_object_editor_set_fill_color (glObjectEditor      *editor,
 						   color_node->key);
 	}
 	
-	g_signal_handlers_unblock_by_func (G_OBJECT(editor->priv->fill_color_combo),
-					   G_CALLBACK (gl_object_editor_changed_cb),
-					   editor);
-	g_signal_handlers_unblock_by_func (G_OBJECT(editor->priv->fill_key_combo),
-					   G_CALLBACK (gl_object_editor_changed_cb),
-					   editor);
+        editor->priv->stop_signals = FALSE;
 
 	gl_debug (DEBUG_EDITOR, "END");
 }
@@ -211,7 +204,9 @@ gl_object_editor_get_fill_color (glObjectEditor      *editor)
 static void
 fill_radio_toggled_cb (glObjectEditor *editor)
 {
-    gl_debug (DEBUG_EDITOR, "START");
+        if (editor->priv->stop_signals) return;
+
+        gl_debug (DEBUG_EDITOR, "START");
 	
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (editor->priv->fill_color_radio))) {
                 gtk_widget_set_sensitive (editor->priv->fill_color_combo, TRUE);
