@@ -1,3 +1,5 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
+
 /*
  *  (GLABELS) Label and Business Card Creation program for GNOME
  *
@@ -23,7 +25,8 @@
 
 #include "prefs-model.h"
 
-#include <libglabels/paper.h>
+#include <libglabels/db.h>
+#include <libglabels/xml.h>
 
 #include "marshal.h"
 #include "util.h"
@@ -69,22 +72,25 @@
 
 #define PREF_MAX_RECENTS                    "/max-recents"
 
+#define PREF_RECENT_TEMPLATES               "/recent-templates"
+#define PREF_MAX_RECENT_TEMPLATES           "/max-recent-templates"
+
 /* Default values */
-#define DEFAULT_UNITS_STRING       units_to_string (GL_UNITS_INCH)
+#define DEFAULT_UNITS_STRING       units_to_string (LGL_UNITS_INCH)
 #define DEFAULT_PAGE_SIZE          "US-Letter"
 
 #define DEFAULT_FONT_FAMILY        "Sans"
 #define DEFAULT_FONT_SIZE          14.0
-#define DEFAULT_FONT_WEIGHT_STRING gl_util_weight_to_string (GNOME_FONT_BOOK)
+#define DEFAULT_FONT_WEIGHT_STRING gl_util_weight_to_string (PANGO_WEIGHT_NORMAL)
 #define DEFAULT_FONT_ITALIC_FLAG   FALSE
-#define DEFAULT_TEXT_JUST_STRING   gl_util_just_to_string (GTK_JUSTIFY_LEFT)
-#define DEFAULT_TEXT_COLOR         GNOME_CANVAS_COLOR (0,0,0)
+#define DEFAULT_TEXT_ALIGN_STRING  gl_util_align_to_string (PANGO_ALIGN_LEFT)
+#define DEFAULT_TEXT_COLOR         GL_COLOR (0,0,0)
 #define DEFAULT_TEXT_LINE_SPACING  1.0
 
 #define DEFAULT_LINE_WIDTH         1.0
-#define DEFAULT_LINE_COLOR         GNOME_CANVAS_COLOR_A (0, 0, 0, 255)
+#define DEFAULT_LINE_COLOR         GL_COLOR_A (0, 0, 0, 255)
 
-#define DEFAULT_FILL_COLOR         GNOME_CANVAS_COLOR_A (0, 255, 0, 255)
+#define DEFAULT_FILL_COLOR         GL_COLOR_A (0, 255, 0, 255)
 
 /*========================================================*/
 /* Private types.                                         */
@@ -101,16 +107,12 @@ enum {
 /* Private globals.                                       */
 /*========================================================*/
 
-static GObjectClass *parent_class = NULL;
-
 static guint signals[LAST_SIGNAL] = {0};
 
 /*========================================================*/
 /* Private function prototypes.                           */
 /*========================================================*/
 
-static void           gl_prefs_model_class_init    (glPrefsModelClass   *klass);
-static void           gl_prefs_model_instance_init (glPrefsModel        *object);
 static void           gl_prefs_model_finalize      (GObject             *object);
 
 static void           notify_cb                    (GConfClient         *client,
@@ -134,48 +136,24 @@ static gdouble        get_float                    (GConfClient         *client,
 						    const gchar         *key,
 						    gdouble              def);
 
-static glUnitsType    string_to_units              (const gchar         *string);
-static const gchar   *units_to_string              (glUnitsType          units);
+static lglUnitsType   string_to_units              (const gchar         *string);
+static const gchar   *units_to_string              (lglUnitsType         units);
 
 
 
 /*****************************************************************************/
 /* Boilerplate object stuff.                                                 */
 /*****************************************************************************/
-GType
-gl_prefs_model_get_type (void)
-{
-	static GType type = 0;
-
-	if (!type) {
-		static const GTypeInfo info = {
-			sizeof (glPrefsModelClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) gl_prefs_model_class_init,
-			NULL,
-			NULL,
-			sizeof (glPrefsModel),
-			0,
-			(GInstanceInitFunc) gl_prefs_model_instance_init,
-			NULL
-		};
-
-		type = g_type_register_static (G_TYPE_OBJECT,
-					       "glPrefsModel", &info, 0);
-	}
-
-	return type;
-}
+G_DEFINE_TYPE (glPrefsModel, gl_prefs_model, G_TYPE_OBJECT);
 
 static void
-gl_prefs_model_class_init (glPrefsModelClass *klass)
+gl_prefs_model_class_init (glPrefsModelClass *class)
 {
-	GObjectClass *object_class = (GObjectClass *) klass;
+	GObjectClass *object_class = G_OBJECT_CLASS (class);
 
 	gl_debug (DEBUG_PREFS, "START");
 
-	parent_class = g_type_class_peek_parent (klass);
+	gl_prefs_model_parent_class = g_type_class_peek_parent (class);
 
 	object_class->finalize = gl_prefs_model_finalize;
 
@@ -193,7 +171,7 @@ gl_prefs_model_class_init (glPrefsModelClass *klass)
 }
 
 static void
-gl_prefs_model_instance_init (glPrefsModel *prefs_model)
+gl_prefs_model_init (glPrefsModel *prefs_model)
 {
 	gl_debug (DEBUG_PREFS, "START");
 
@@ -217,16 +195,17 @@ gl_prefs_model_instance_init (glPrefsModel *prefs_model)
 static void
 gl_prefs_model_finalize (GObject *object)
 {
+	glPrefsModel *prefs_model = GL_PREFS_MODEL (object);
+
 	gl_debug (DEBUG_PREFS, "START");
 
 	g_return_if_fail (object && GL_IS_PREFS_MODEL (object));
 
-	g_object_unref (G_OBJECT(GL_PREFS_MODEL(object)->gconf_client));
+	g_object_unref (G_OBJECT(prefs_model->gconf_client));
+	g_free (prefs_model->default_page_size);
+	g_free (prefs_model->default_font_family);
 
-	g_free (GL_PREFS_MODEL(object)->default_page_size);
-	g_free (GL_PREFS_MODEL(object)->default_font_family);
-
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (gl_prefs_model_parent_class)->finalize (object);
 
 	gl_debug (DEBUG_PREFS, "END");
 }
@@ -269,6 +248,7 @@ gl_prefs_model_save_settings (glPrefsModel *prefs_model)
 				 BASE_KEY PREF_UNITS,
 				 units_to_string(prefs_model->units),
 				 NULL);
+        lgl_xml_set_default_units (prefs_model->units);
 
 	/* Default page size */
 	gconf_client_set_string (prefs_model->gconf_client,
@@ -300,7 +280,7 @@ gl_prefs_model_save_settings (glPrefsModel *prefs_model)
 
 	gconf_client_set_string (prefs_model->gconf_client,
 				 BASE_KEY PREF_DEFAULT_TEXT_ALIGNMENT,
-				 gl_util_just_to_string(prefs_model->default_text_alignment),
+				 gl_util_align_to_string(prefs_model->default_text_alignment),
 				 NULL);
 
 	gconf_client_set_float  (prefs_model->gconf_client,
@@ -382,6 +362,17 @@ gl_prefs_model_save_settings (glPrefsModel *prefs_model)
 			      prefs_model->max_recents,
 			      NULL);
 
+	/* Recent templates */
+	gconf_client_set_list (prefs_model->gconf_client,
+			       BASE_KEY PREF_RECENT_TEMPLATES,
+                               GCONF_VALUE_STRING,
+                               prefs_model->recent_templates,
+                               NULL);
+	gconf_client_set_int (prefs_model->gconf_client,
+			      BASE_KEY PREF_MAX_RECENT_TEMPLATES,
+			      prefs_model->max_recent_templates,
+			      NULL);
+
 
 	gconf_client_suggest_sync (prefs_model->gconf_client, NULL);
 	
@@ -394,8 +385,9 @@ gl_prefs_model_save_settings (glPrefsModel *prefs_model)
 void
 gl_prefs_model_load_settings (glPrefsModel *prefs_model)
 {
-	gchar   *string;
-	glPaper *paper;
+	gchar    *string;
+	lglPaper *paper;
+        GSList   *p, *p_next;
 
 	gl_debug (DEBUG_PREFS, "START");
 	
@@ -409,15 +401,18 @@ gl_prefs_model_load_settings (glPrefsModel *prefs_model)
 			    DEFAULT_UNITS_STRING);
 	prefs_model->units = string_to_units( string );
 	g_free( string );
+        lgl_xml_set_default_units (prefs_model->units);
 
 
 	/* Page size */
+        g_free (prefs_model->default_page_size);
 	prefs_model->default_page_size =
 		get_string (prefs_model->gconf_client,
 			    BASE_KEY PREF_DEFAULT_PAGE_SIZE,
 			    DEFAULT_PAGE_SIZE);
 
 	/* Text properties */
+        g_free (prefs_model->default_font_family);
 	prefs_model->default_font_family =
 		get_string (prefs_model->gconf_client,
 			    BASE_KEY PREF_DEFAULT_FONT_FAMILY,
@@ -443,8 +438,8 @@ gl_prefs_model_load_settings (glPrefsModel *prefs_model)
 	string =
 		get_string (prefs_model->gconf_client,
 			    BASE_KEY PREF_DEFAULT_TEXT_ALIGNMENT,
-			    DEFAULT_TEXT_JUST_STRING);
-	prefs_model->default_text_alignment = gl_util_string_to_just( string );
+			    DEFAULT_TEXT_ALIGN_STRING);
+	prefs_model->default_text_alignment = gl_util_string_to_align( string );
 	g_free( string );
 
 	prefs_model->default_text_line_spacing =
@@ -525,20 +520,48 @@ gl_prefs_model_load_settings (glPrefsModel *prefs_model)
 	prefs_model->max_recents =
 		get_int (prefs_model->gconf_client,
 			 BASE_KEY PREF_MAX_RECENTS,
-			 4);
+			 -1);
+
+	/* Recent templates */
+        for (p=prefs_model->recent_templates; p != NULL; p=p->next)
+        {
+                g_free (p->data);
+        }
+        g_slist_free (prefs_model->recent_templates);
+	prefs_model->recent_templates =
+		gconf_client_get_list (prefs_model->gconf_client,
+                                       BASE_KEY PREF_RECENT_TEMPLATES,
+                                       GCONF_VALUE_STRING,
+                                       NULL);
+	prefs_model->max_recent_templates =
+		get_int (prefs_model->gconf_client,
+			 BASE_KEY PREF_MAX_RECENT_TEMPLATES,
+			 5);
 
 
 	/* Proof read the default page size -- it must be a valid id. */
 	/* (For compatability with older versions.) */
-	/* Note: paper module must be initialized for this to work. */
-	paper = gl_paper_from_id (prefs_model->default_page_size);
+	paper = lgl_db_lookup_paper_from_id (prefs_model->default_page_size);
 	if ( paper == NULL ) {
 		prefs_model->default_page_size = g_strdup (DEFAULT_PAGE_SIZE);
 	} else {
-		gl_paper_free (paper);
+		lgl_paper_free (paper);
 		paper = NULL;
 	}
 
+        /* Proof read the recent templates list.  Make sure the template names */
+        /* are valid.  Remove from list if not. */
+        for (p=prefs_model->recent_templates; p != NULL; p=p_next)
+        {
+                p_next = p->next;
+
+                if ( !lgl_db_does_template_name_exist (p->data) )
+                {
+                        g_free (p->data);
+                        prefs_model->recent_templates = g_slist_delete_link (prefs_model->recent_templates, p);
+                }
+        }
+
 	gl_debug (DEBUG_PREFS, "max_recents = %d", prefs_model->max_recents);
 
 
@@ -673,35 +696,35 @@ get_float (GConfClient *client,
 /*---------------------------------------------------------------------------*/
 /* PRIVATE.  Utilities to deal with units.                                   */
 /*---------------------------------------------------------------------------*/
-static glUnitsType
+static lglUnitsType
 string_to_units (const gchar *string)
 {
-	glUnitsType units;
+	lglUnitsType units;
 
 	if (g_strcasecmp (string, "Points") == 0) {
-		units = GL_UNITS_POINT;
+		units = LGL_UNITS_POINT;
 	} else if (g_strcasecmp (string, "Inches") == 0) {
-		units = GL_UNITS_INCH;
+		units = LGL_UNITS_INCH;
 	} else if (g_strcasecmp (string, "Millimeters") == 0) {
-		units = GL_UNITS_MM;
+		units = LGL_UNITS_MM;
 	} else {
-		units = GL_UNITS_INCH;
+		units = LGL_UNITS_INCH;
 	}
 
 	return units;
 }
 
 static const
-gchar *units_to_string (glUnitsType units)
+gchar *units_to_string (lglUnitsType units)
 {
 	switch (units) {
-	case GL_UNITS_POINT:
+	case LGL_UNITS_POINT:
 		return "Points";
 		break;
-	case GL_UNITS_INCH:
+	case LGL_UNITS_INCH:
 		return "Inches";
 		break;
-	case GL_UNITS_MM:
+	case LGL_UNITS_MM:
 		return "Millimeters";
 		break;
 	default:
