@@ -3,9 +3,9 @@
 /*
  *  (GLABELS) Label and Business Card Creation program for GNOME
  *
- *  view_line.c:  GLabels label line object widget
+ *  view_line.c:  GLabels label line object view
  *
- *  Copyright (C) 2001-2006  Jim Evins <evins@snaught.com>.
+ *  Copyright (C) 2001-2007  Jim Evins <evins@snaught.com>.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,7 +28,6 @@
 #include <glib/gi18n.h>
 #include <glib/gmem.h>
 
-#include "view-highlight.h"
 #include "color.h"
 #include "object-editor.h"
 #include "stock.h"
@@ -42,35 +41,27 @@
 /* Private macros and constants.                          */
 /*========================================================*/
 
+
 /*========================================================*/
 /* Private types.                                         */
 /*========================================================*/
 
 struct _glViewLinePrivate {
-	GnomeCanvasItem       *object_item;
-	GnomeCanvasItem       *shadow_item;
 };
 
 /*========================================================*/
 /* Private globals.                                       */
 /*========================================================*/
 
-static glViewObjectClass *parent_class = NULL;
-
 
 /*========================================================*/
 /* Private function prototypes.                           */
 /*========================================================*/
 
-static void       gl_view_line_class_init           (glViewLineClass  *klass);
-static void       gl_view_line_instance_init        (glViewLine       *view_line);
 static void       gl_view_line_finalize             (GObject          *object);
 
 static GtkWidget *construct_properties_editor       (glViewObject     *view_object);
 
-static void       update_canvas_item_from_object_cb (glLabelObject    *object,
-						     glViewLine        *view_line);
-
 static void       update_object_from_editor_cb      (glObjectEditor   *editor,
 						     glLabelObject    *object);
 
@@ -82,63 +73,43 @@ static void       update_editor_from_move_cb        (glLabelObject    *object,
 						     gdouble           dy,
 						     glObjectEditor   *editor);
 
-static void       update_editor_from_label_cb       (glLabel          *label,
-						     glObjectEditor   *editor);
+static gboolean   object_at                         (glViewObject     *view_object,
+                                                     cairo_t          *cr,
+                                                     gdouble           x,
+                                                     gdouble           y);
 
 
 
 /*****************************************************************************/
 /* Boilerplate object stuff.                                                 */
 /*****************************************************************************/
-GType
-gl_view_line_get_type (void)
-{
-	static GType type = 0;
+G_DEFINE_TYPE (glViewLine, gl_view_line, GL_TYPE_VIEW_OBJECT);
 
-	if (!type) {
-		static const GTypeInfo info = {
-			sizeof (glViewLineClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) gl_view_line_class_init,
-			NULL,
-			NULL,
-			sizeof (glViewLine),
-			0,
-			(GInstanceInitFunc) gl_view_line_instance_init,
-			NULL
-		};
-
-		type = g_type_register_static (GL_TYPE_VIEW_OBJECT,
-					       "glViewLine", &info, 0);
-	}
-
-	return type;
-}
 
 static void
-gl_view_line_class_init (glViewLineClass *klass)
+gl_view_line_class_init (glViewLineClass *class)
 {
-	GObjectClass      *object_class      = (GObjectClass *) klass;
-	glViewObjectClass *view_object_class = (glViewObjectClass *) klass;
+	GObjectClass      *object_class      = G_OBJECT_CLASS (class);
+	glViewObjectClass *view_object_class = GL_VIEW_OBJECT_CLASS (class);
 
 	gl_debug (DEBUG_VIEW, "START");
 
-	parent_class = g_type_class_peek_parent (klass);
+	gl_view_line_parent_class = g_type_class_peek_parent (class);
 
 	object_class->finalize = gl_view_line_finalize;
 
 	view_object_class->construct_editor = construct_properties_editor;
+	view_object_class->object_at        = object_at;
 
 	gl_debug (DEBUG_VIEW, "END");
 }
 
 static void
-gl_view_line_instance_init (glViewLine *view_line)
+gl_view_line_init (glViewLine *view_line)
 {
 	gl_debug (DEBUG_VIEW, "START");
 
-	view_line->private = g_new0 (glViewLinePrivate, 1);
+	view_line->priv = g_new0 (glViewLinePrivate, 1);
 
 	gl_debug (DEBUG_VIEW, "END");
 }
@@ -146,51 +117,39 @@ gl_view_line_instance_init (glViewLine *view_line)
 static void
 gl_view_line_finalize (GObject *object)
 {
-	glLabel       *parent;
+        glViewLine *view_line = GL_VIEW_LINE (object);
 
 	gl_debug (DEBUG_VIEW, "START");
 
 	g_return_if_fail (object && GL_IS_VIEW_LINE (object));
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+        g_free (view_line->priv);
+
+	G_OBJECT_CLASS (gl_view_line_parent_class)->finalize (object);
 
 	gl_debug (DEBUG_VIEW, "END");
 }
 
 /*****************************************************************************/
-/* NEW line object view.                                                     */
+/* NEW line object view.                                                  */
 /*****************************************************************************/
 glViewObject *
 gl_view_line_new (glLabelLine *object,
-		  glView      *view)
+                  glView     *view)
 {
-	glViewLine        *view_line;
+	glViewLine         *view_line;
 
 	gl_debug (DEBUG_VIEW, "START");
 
-	g_return_if_fail (object && GL_IS_LABEL_LINE (object));
-	g_return_if_fail (view && GL_IS_VIEW (view));
+	g_return_val_if_fail (object && GL_IS_LABEL_LINE (object), NULL);
+	g_return_val_if_fail (view && GL_IS_VIEW (view), NULL);
 	
 	view_line = g_object_new (gl_view_line_get_type(), NULL);
 
-	gl_view_object_set_view (GL_VIEW_OBJECT(view_line), view);
 	gl_view_object_set_object (GL_VIEW_OBJECT(view_line),
 				   GL_LABEL_OBJECT(object),
-				   GL_VIEW_HIGHLIGHT_LINE_RESIZABLE);
-
-	/* Create analogous canvas item. */
-	view_line->private->shadow_item =
-		gl_view_object_item_new (GL_VIEW_OBJECT(view_line),
-					 gnome_canvas_line_get_type (),
-					 NULL);
-	view_line->private->object_item =
-		gl_view_object_item_new (GL_VIEW_OBJECT(view_line),
-					 gnome_canvas_line_get_type (),
-					 NULL);
-	update_canvas_item_from_object_cb (GL_LABEL_OBJECT(object), view_line);
-
-	g_signal_connect (G_OBJECT (object), "changed",
-			  G_CALLBACK (update_canvas_item_from_object_cb), view_line);
+				   GL_VIEW_OBJECT_HANDLES_LINE);
+	gl_view_object_set_view (GL_VIEW_OBJECT(view_line), view);
 
 	gl_debug (DEBUG_VIEW, "END");
 
@@ -198,7 +157,7 @@ gl_view_line_new (glLabelLine *object,
 }
 
 /*****************************************************************************/
-/* Create a properties editor for a line object.                             */
+/* Create a properties dialog for a line object.                             */
 /*****************************************************************************/
 static GtkWidget *
 construct_properties_editor (glViewObject *view_object)
@@ -213,14 +172,14 @@ construct_properties_editor (glViewObject *view_object)
 
 	/* Build editor. */
 	editor = gl_object_editor_new (GL_STOCK_LINE, _("Line object properties"),
+                                       object->parent,
+				       GL_OBJECT_EDITOR_SHADOW_PAGE,
 				       GL_OBJECT_EDITOR_POSITION_PAGE,
 				       GL_OBJECT_EDITOR_SIZE_LINE_PAGE,
 				       GL_OBJECT_EDITOR_LINE_PAGE,
-				       GL_OBJECT_EDITOR_SHADOW_PAGE,
 				       0);
 	
 	/* Update */
-	update_editor_from_label_cb (object->parent, GL_OBJECT_EDITOR(editor));
 	update_editor_from_object_cb (object, GL_OBJECT_EDITOR(editor));
 	update_editor_from_move_cb (object, 0, 0, GL_OBJECT_EDITOR(editor));
 
@@ -231,94 +190,12 @@ construct_properties_editor (glViewObject *view_object)
 			  G_CALLBACK (update_editor_from_object_cb), editor);
 	g_signal_connect (G_OBJECT (object), "moved",
 			  G_CALLBACK (update_editor_from_move_cb), editor);
-	g_signal_connect (G_OBJECT (object->parent), "size_changed",
-			  G_CALLBACK (update_editor_from_label_cb), editor);
-	g_signal_connect (G_OBJECT (object->parent), "merge_changed",
-			  G_CALLBACK (update_editor_from_label_cb), editor);
 
 	gl_debug (DEBUG_VIEW, "END");
 
 	return editor;
 }
 
-/*---------------------------------------------------------------------------*/
-/* PRIVATE. label object "changed" callback.                                 */
-/*---------------------------------------------------------------------------*/
-static void
-update_canvas_item_from_object_cb (glLabelObject *object,
-				   glViewLine     *view_line)
-{
-	gdouble            line_width;
-	glColorNode       *line_color_node;
-	gdouble            w, h;
-	GnomeCanvasPoints *points;
-	gboolean           shadow_state;
-	gdouble            shadow_x, shadow_y;
-	glColorNode	  *shadow_color_node;
-	gdouble            shadow_opacity;
-	guint              shadow_line_color;
-
-	gl_debug (DEBUG_VIEW, "START");
-
-	/* Query properties of object. */
-	gl_label_object_get_size (GL_LABEL_OBJECT(object), &w, &h);
-	line_width = gl_label_object_get_line_width(GL_LABEL_OBJECT(object));
-	line_color_node = gl_label_object_get_line_color(GL_LABEL_OBJECT(object));
-	if (line_color_node->field_flag)
-	{
-		line_color_node->color = GL_COLOR_MERGE_DEFAULT;
-	}
-	shadow_state = gl_label_object_get_shadow_state (GL_LABEL_OBJECT (object));
-	gl_label_object_get_shadow_offset (GL_LABEL_OBJECT (object), &shadow_x, &shadow_y);
-	shadow_color_node = gl_label_object_get_shadow_color (GL_LABEL_OBJECT (object));
-	if (shadow_color_node->field_flag)
-	{
-		shadow_color_node->color = GL_COLOR_SHADOW_MERGE_DEFAULT;
-	}
-	shadow_opacity = gl_label_object_get_shadow_opacity (GL_LABEL_OBJECT (object));
-	shadow_line_color = gl_color_shadow (shadow_color_node->color,
-					     shadow_opacity,
-					     line_color_node->color);
-
-	/* Adjust appearance of analogous canvas item. */
-	points = gnome_canvas_points_new (2);
-	points->coords[0] = shadow_x;
-	points->coords[1] = shadow_y;
-	points->coords[2] = shadow_x + w;
-	points->coords[3] = shadow_y + h;
-	gnome_canvas_item_set (view_line->private->shadow_item,
-			       "points", points,
-			       "width_units", line_width,
-			       "fill_color_rgba", shadow_line_color,
-			       NULL);
-
-	if (shadow_state)
-	{
-		gnome_canvas_item_show (view_line->private->shadow_item);
-	}
-	else
-	{
-		gnome_canvas_item_hide (view_line->private->shadow_item);
-	}
-
-	points->coords[0] = 0.0;
-	points->coords[1] = 0.0;
-	points->coords[2] = w;
-	points->coords[3] = h;
-	gnome_canvas_item_set (view_line->private->object_item,
-			       "points", points,
-			       "width_units", line_width,
-			       "fill_color_rgba", line_color_node->color,
-			       NULL);
-
-	gnome_canvas_points_free (points);
-
-	gl_color_node_free (&line_color_node);
-	gl_color_node_free (&shadow_color_node);
-
-	gl_debug (DEBUG_VIEW, "END");
-}
-
 /*---------------------------------------------------------------------------*/
 /* PRIVATE.  editor "changed" callback.                                      */
 /*---------------------------------------------------------------------------*/
@@ -333,6 +210,7 @@ update_object_from_editor_cb (glObjectEditor *editor,
 	gdouble            shadow_x, shadow_y;
 	glColorNode	  *shadow_color_node;
 	gdouble            shadow_opacity;
+	
 
 	gl_debug (DEBUG_VIEW, "START");
 
@@ -343,13 +221,12 @@ update_object_from_editor_cb (glObjectEditor *editor,
 					 update_editor_from_move_cb,
 					 editor);
 
-
 	gl_object_editor_get_position (editor, &x, &y);
 	gl_label_object_set_position (object, x, y);
 
 	gl_object_editor_get_lsize (editor, &w, &h);
 	gl_label_object_set_size (object, w, h);
-	
+
 	line_color_node = gl_object_editor_get_line_color (editor);
 	gl_label_object_set_line_color (object, line_color_node);
 	gl_color_node_free (&line_color_node);
@@ -370,7 +247,6 @@ update_object_from_editor_cb (glObjectEditor *editor,
 	shadow_opacity = gl_object_editor_get_shadow_opacity (editor);
 	gl_label_object_set_shadow_opacity (object, shadow_opacity);
 
-
 	g_signal_handlers_unblock_by_func (G_OBJECT(object),
 					   update_editor_from_object_cb,
 					   editor);
@@ -401,9 +277,8 @@ update_editor_from_object_cb (glLabelObject  *object,
 
 	gl_label_object_get_size (object, &w, &h);
 	gl_object_editor_set_lsize (editor, w, h);
-	
 	merge = gl_label_get_merge (GL_LABEL(object->parent));
-
+	
 	line_color_node = gl_label_object_get_line_color (GL_LABEL_OBJECT(object));
 	gl_object_editor_set_line_color (editor, (merge != NULL), line_color_node);
 	gl_color_node_free (&line_color_node);
@@ -446,59 +321,61 @@ update_editor_from_move_cb (glLabelObject    *object,
 	gl_debug (DEBUG_VIEW, "END");
 }
 
-/*---------------------------------------------------------------------------*/
-/* PRIVATE. label "changed" callback.                                        */
-/*---------------------------------------------------------------------------*/
-static void
-update_editor_from_label_cb (glLabel        *label,
-			     glObjectEditor *editor)
+/*****************************************************************************/
+/* Is object at (x,y)?                                                       */
+/*****************************************************************************/
+static gboolean
+object_at (glViewObject  *view_object,
+           cairo_t       *cr,
+           gdouble        x,
+           gdouble        y)
 {
-	gdouble            label_width, label_height;
-	glMerge     	  *merge;
+	glLabelObject    *object;
+        gdouble           w, h;
+        gdouble           line_width;
 
-	gl_debug (DEBUG_VIEW, "START");
+        object = gl_view_object_get_object (view_object);
 
-	gl_label_get_size (label, &label_width, &label_height);
-	gl_object_editor_set_max_position (GL_OBJECT_EDITOR (editor),
-					   label_width, label_height);
-	gl_object_editor_set_max_lsize (GL_OBJECT_EDITOR (editor),
-					label_width, label_height);
-	gl_object_editor_set_max_shadow_offset (GL_OBJECT_EDITOR (editor),
-						label_width, label_height);
-	
-	merge = gl_label_get_merge (label);
-	gl_object_editor_set_key_names (editor, merge);
+        gl_label_object_get_size (object, &w, &h);
 
-	gl_debug (DEBUG_VIEW, "END");
+        cairo_move_to (cr, 0.0, 0.0);
+        cairo_line_to (cr, w, h);
+
+        line_width = gl_label_object_get_line_width (object);
+        cairo_set_line_width (cr, line_width);
+        if (cairo_in_stroke (cr, x, y))
+        {
+                return TRUE;
+        }
+
+        return FALSE;
 }
 
+
 /*****************************************************************************/
 /* Return apropos cursor for create object mode.                             */
 /*****************************************************************************/
 GdkCursor *
 gl_view_line_get_create_cursor (void)
 {
-	static GdkCursor *cursor = NULL;
-	GdkPixmap        *pixmap_data, *pixmap_mask;
+	GdkCursor       *cursor = NULL;
+	GdkPixmap       *pixmap_data, *pixmap_mask;
 	GdkColor         fg = { 0, 0, 0, 0 };
 	GdkColor         bg = { 0, 65535, 65535, 65535 };
 
 	gl_debug (DEBUG_VIEW, "START");
 
-	if (!cursor) {
-		pixmap_data = gdk_bitmap_create_from_data (NULL,
-							   (gchar *)cursor_line_bits,
-							   cursor_line_width,
-							   cursor_line_height);
-		pixmap_mask = gdk_bitmap_create_from_data (NULL,
-							   (gchar *)cursor_line_mask_bits,
-							   cursor_line_mask_width,
-							   cursor_line_mask_height);
-		cursor =
-		    gdk_cursor_new_from_pixmap (pixmap_data, pixmap_mask, &fg,
-						&bg, cursor_line_x_hot,
-						cursor_line_y_hot);
-	}
+        pixmap_data = gdk_bitmap_create_from_data (NULL,
+                                                   (gchar *)cursor_line_bits,
+                                                   cursor_line_width,
+                                                   cursor_line_height);
+        pixmap_mask = gdk_bitmap_create_from_data (NULL,
+                                                   (gchar *)cursor_line_mask_bits,
+                                                   cursor_line_mask_width,
+                                                   cursor_line_mask_height);
+        cursor = gdk_cursor_new_from_pixmap (pixmap_data, pixmap_mask, &fg,
+                                             &bg, cursor_line_x_hot,
+                                             cursor_line_y_hot);
 
 	gl_debug (DEBUG_VIEW, "END");
 
@@ -506,104 +383,73 @@ gl_view_line_get_create_cursor (void)
 }
 
 /*****************************************************************************/
-/* Canvas event handler for creating line objects.                            */
+/* Object creation handler: button press event.                              */
 /*****************************************************************************/
-int
-gl_view_line_create_event_handler (GnomeCanvas *canvas,
-				   GdkEvent    *event,
-				   glView      *view)
+void
+gl_view_line_create_button_press_event   (glView *view,
+                                          gdouble x,
+                                          gdouble y)
 {
-	static gdouble       x0, y0;
-	static gboolean      dragging = FALSE;
-	static glViewObject *view_line;
-	static GObject      *object;
-	gdouble              x, y, w, h;
+	GObject             *object;
 	glColorNode         *line_color_node;
 
-	gl_debug (DEBUG_VIEW, "");
+        gl_view_unselect_all (view);
 
-	switch (event->type) {
+        line_color_node = gl_color_node_new_default ();
+		
+        object = gl_label_line_new (view->label);
+        gl_label_object_set_position (GL_LABEL_OBJECT(object), x, y);
+        gl_label_object_set_size (GL_LABEL_OBJECT(object), 0.0, 0.0);
+        line_color_node->color = gl_color_set_opacity (gl_view_get_default_line_color(view), 0.5);
+        gl_label_object_set_line_width (GL_LABEL_OBJECT(object),
+                                        gl_view_get_default_line_width(view));
+        gl_label_object_set_line_color (GL_LABEL_OBJECT(object),
+                                        line_color_node);
 
-	case GDK_BUTTON_PRESS:
-		switch (event->button.button) {
-		case 1:
-			line_color_node = gl_color_node_new_default ();
-			dragging = TRUE;
-			gnome_canvas_item_grab (canvas->root,
-						GDK_POINTER_MOTION_MASK |
-						GDK_BUTTON_RELEASE_MASK |
-						GDK_BUTTON_PRESS_MASK,
-						NULL, event->button.time);
-			gnome_canvas_window_to_world (canvas,
-						      event->button.x,
-						      event->button.y, &x, &y);
-			object = gl_label_line_new (view->label);
-			gl_label_object_set_position (GL_LABEL_OBJECT(object),
-						      x, y);
-			gl_label_object_set_size (GL_LABEL_OBJECT(object),
-						  0.0, 0.0);
-			gl_label_object_set_line_width (GL_LABEL_OBJECT(object),
-						      gl_view_get_default_line_width(view));
-			line_color_node->color = gl_color_set_opacity (gl_view_get_default_line_color(view), 0.5);
-			gl_label_object_set_line_color (GL_LABEL_OBJECT(object),
-						     line_color_node);
-			view_line = gl_view_line_new (GL_LABEL_LINE(object),
-						      view);
-			x0 = x;
-			y0 = y;
-			gl_color_node_free (&line_color_node);
-			return TRUE;
-
-		default:
-			return FALSE;
-		}
-
-	case GDK_BUTTON_RELEASE:
-		switch (event->button.button) {
-		case 1:
-			line_color_node = gl_color_node_new_default ();
-			dragging = FALSE;
-			gnome_canvas_item_ungrab (canvas->root, event->button.time);
-			gnome_canvas_window_to_world (canvas,
-						      event->button.x,
-						      event->button.y, &x, &y);
-			if ((x0 == x) && (y0 == y)) {
-				x = x0 + 36.0;
-				y = y0 + 36.0;
-			}
-			w = x - x0;
-			h = y - y0;
-			gl_label_object_set_size (GL_LABEL_OBJECT(object),
-						  w, h);
-			line_color_node->color = gl_view_get_default_line_color(view);
-			gl_label_object_set_line_color (GL_LABEL_OBJECT(object),
-						     line_color_node);
-			gl_view_unselect_all (view);
-			gl_view_object_select (GL_VIEW_OBJECT(view_line));
-			gl_view_arrow_mode (view);
-			gl_color_node_free (&line_color_node);
-			return TRUE;
-
-		default:
-			return FALSE;
-		}
-
-	case GDK_MOTION_NOTIFY:
-		if (dragging && (event->motion.state & GDK_BUTTON1_MASK)) {
-			gnome_canvas_window_to_world (canvas,
-						      event->motion.x,
-						      event->motion.y, &x, &y);
-			w = x - x0;
-			h = y - y0;
-			gl_label_object_set_size (GL_LABEL_OBJECT(object),
-						  w, h);
-			return TRUE;
-		} else {
-			return FALSE;
-		}
-
-	default:
-		return FALSE;
-	}
+        gl_color_node_free (&line_color_node);
 
+        view->create_object = GL_LABEL_OBJECT (object);
+        view->create_x0 = x;
+        view->create_y0 = y;
 }
+
+/*****************************************************************************/
+/* Object creation handler: motion event.                                    */
+/*****************************************************************************/
+void
+gl_view_line_create_motion_event     (glView *view,
+                                      gdouble x,
+                                      gdouble y)
+{
+        gdouble w, h;
+
+        w = x - view->create_x0;
+        h = y - view->create_y0;
+        gl_label_object_set_size (GL_LABEL_OBJECT(view->create_object), w, h);
+}
+
+/*****************************************************************************/
+/* Object creation handler: button relesase event.                           */
+/*****************************************************************************/
+void
+gl_view_line_create_button_release_event (glView *view,
+                                          gdouble x,
+                                          gdouble y)
+{
+	glColorNode         *line_color_node;
+        gdouble              w, h;
+
+        line_color_node = gl_color_node_new_default ();
+		
+        if ((view->create_x0 == x) && (view->create_y0 == y)) {
+                x = view->create_x0 + 36.0;
+                y = view->create_y0 + 36.0;
+        }
+        w = x - view->create_x0;
+        h = y - view->create_y0;
+        gl_label_object_set_size (GL_LABEL_OBJECT(view->create_object), w, h);
+        line_color_node->color = gl_view_get_default_line_color(view);
+        gl_label_object_set_line_color (GL_LABEL_OBJECT(view->create_object), line_color_node);
+        gl_color_node_free (&line_color_node);
+}
+
